#include <caml/alloc.h>
#include <caml/signals.h>
#include <unistd.h>
#include "libgraph.h"
#import <Cocoa/Cocoa.h>
#import "MC.h"

extern NSLock* lock;
extern NSMutableArray* queue;
extern id <ServerProtocol> mcProxy;
extern NSString* serverName;
extern int tailleImage;
extern unsigned char* data;

value caml_gr_wait_event(value eventlist)
{
	caml_gr_check_open();

	int mask = 0;
	BOOL poll = NO;
    NSDictionary* mce = @{
        @"mask" : @0,
        @"mouse_x" : @-1,
        @"mouse_y" : @-1,
        @"key" : @0
    };

    while (eventlist != Val_int(0)) {
        switch (Int_val(Field(eventlist, 0))) {
            case 0:                     // Button_down
                mask |= MOUSEDOWN; break;
            case 1:                     // Button_up
                mask |= MOUSEUP; break;
            case 2:                     // Key_pressed
                mask |= KEYDOWN; break;
            case 3:                     // Mouse_motion
                mask |=MOUSEMOVE; break;
            case 4:                     // Poll
                poll = YES; break;
        }
        eventlist = Field(eventlist, 1);
    }

	if (poll)
	{
        [lock lock];
        int n = 0;
        if (queue != nil && (n = [queue count]))
        {
            int i = 0;
            while (i < n)
            {
                mce = [queue objectAtIndex:i];
                if (((int)[mce[@"mask"] intValue] & KEYDOWN) != 0)
                {
                    break;
                }
                i++;
            }
        }
        [lock unlock];
	}
	else
	{
        while (((int)[mce[@"mask"] intValue] & mask) == 0)
		{
			caml_gr_check_open();
			usleep(20);
            
			caml_enter_blocking_section();
			caml_leave_blocking_section();
            
			[lock lock];
			int n = 0;
			if (queue != nil && (n =[queue count]))
			{
				int i = 0;
				while (i < n)
				{
                    mce = [queue objectAtIndex:i];
                    if (((int)[mce[@"mask"] intValue] & mask) != 0)
					{
						[queue removeObjectAtIndex:i];
						break;
					}
					i++;
				}
			}
			[lock unlock];
		}
	}
    if ((int)[mce[@"mask"] intValue] & MOUSEMOVE)
    {
        [mcProxy getImage:serverName im:[NSData dataWithBytes: data length: tailleImage]];
    }
	value res = caml_alloc_tuple(5);
    Field(res, 0) = Val_int((int)[mce[@"mouse_x"] intValue]);
    Field(res, 1) = Val_int((int)[mce[@"mouse_y"] intValue]);
    Field(res, 2) = Val_bool((int)[mce[@"mask"] intValue] & (MOUSEDOWN | MOUSEMOVE));
    Field(res, 3) = Val_bool((int)[mce[@"mask"] intValue] & KEYDOWN);
    Field(res, 4) = Val_int((int)[mce[@"key"] intValue] & 0xFF);
	return res;
}
