#import <Cocoa/Cocoa.h>
#include "libgraph.h"
#include <caml/alloc.h>

extern float fontSize;
extern char fontName[255];
extern int grx, gry;
extern BOOL needsRefresh;
extern float red, green, blue;

static void caml_gr_font(char *fontname)
{
    @autoreleasepool {
        NSArray* availableFonts = [[NSFontManager sharedFontManager] availableFonts];
        NSEnumerator* enumerator = [availableFonts objectEnumerator];
        NSString* str;
        NSString* ss = [NSString stringWithCString:String_val(fontname) encoding: NSISOLatin1StringEncoding];
        while (str = [enumerator nextObject])
        {
            if ([str hasPrefix: ss])
            {
                [[NSFont fontWithName:str size:fontSize] set];
                strcpy(fontName, [str cStringUsingEncoding: NSISOLatin1StringEncoding]);
                return;
            }
        }
    }
    return;
    
}

static int text_length(char * str)
{
    NSUInteger lenA = (NSUInteger)strlen(str);
    NSString* s = [[NSString allocWithZone:nil] initWithBytes:str length:lenA encoding: NSUTF8StringEncoding];
    NSUInteger len = s ? [s length] : lenA;
    return  len;
}

value caml_gr_text_length(value str)
{
    return Val_int(text_length(String_val(str)));
}

value caml_gr_set_font(value fontname)
{
    caml_gr_check_open();
    caml_gr_font(String_val(fontname));
    return Val_unit;
}

value caml_gr_set_text_size (value sz)
{
    caml_gr_check_open();
    fontSize = Int_val(sz);
    @autoreleasepool {
        NSFont* font = [NSFont fontWithName:[NSString stringWithCString: fontName encoding: NSUTF8StringEncoding] size: fontSize];
        [font set];
    }
    return Val_unit;;
}

static void caml_gr_draw_text(char *txt, int len) // len unused...
{
    @autoreleasepool {
        NSFont* font = [NSFont fontWithName: [NSString stringWithCString: fontName encoding: NSISOLatin1StringEncoding] size: fontSize];
        NSColor* color = [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0];
        NSDictionary* dico = [NSDictionary dictionaryWithObjects:
                              [NSArray arrayWithObjects:font, color, nil]
                                                         forKeys:
                              [NSArray arrayWithObjects:NSFontAttributeName, NSForegroundColorAttributeName, nil]];
        
        int width = [[NSString stringWithCString:txt encoding: NSUTF8StringEncoding] sizeWithAttributes: dico].width;
        [[NSString stringWithCString:txt encoding: NSUTF8StringEncoding] drawAtPoint: NSMakePoint(grx, gry) withAttributes: dico];
        grx += width;
    }
    needsRefresh = YES;
    
}

value caml_gr_draw_char(value chr)
{
    char str[2];
    str[0] = Int_val(chr);
    str[1] = '\0';
    caml_gr_draw_text(str, 0);
    return Val_unit;
    
}

value caml_gr_draw_string(value str)
{
    caml_gr_check_open();
    caml_gr_draw_text(String_val(str), 0);
    return Val_unit;
}

value caml_gr_text_size(value str)
{
    caml_gr_check_open();
    NSSize size = NSMakeSize(0, 0);
    @autoreleasepool {
        NSFont* font = [NSFont fontWithName:[NSString stringWithCString: fontName encoding: NSISOLatin1StringEncoding] size: fontSize];
        size = [[NSString stringWithCString:String_val(str) encoding: NSUTF8StringEncoding]
                       sizeWithAttributes:
                       [NSDictionary dictionaryWithObjects:
                        [NSArray arrayWithObjects:font, [NSColor blackColor], nil]
                                                   forKeys:
                        [NSArray arrayWithObjects:NSFontAttributeName, NSForegroundColorAttributeName, nil]]];
    }
    value res = caml_alloc_small(2, 0);
    int width = size.width + 0.5;
    int height = size.height + 0.5;
    Field(res, 0) = Val_int(width);
    Field(res, 1) = Val_int(height);
    return res;
}
